<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Models\User;
use App\Models\Wallet;
use App\Models\SubWallet;
use Illuminate\Auth\Events\Registered;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Validation\Rules;
use App\Helpers\WalletHelper;
use App\Models\Transaction;
use Illuminate\Http\JsonResponse;
use Illuminate\Support\Str;

class RegisterController extends Controller
{
    public function registerDonor(Request $request)
    {
        return $this->registerViaApi($request, 'donor');
    }

    public function registerBeneficiary(Request $request)
    {
        return $this->registerViaApi($request, 'beneficiary');
    }

    private function registerViaApi(Request $request, string $role)
    {
        try {
            // Common validation
            $baseRules = [
                'email' => 'required|email|unique:users',
                'phone' => 'required|string|unique:users',
                'country_code' => 'required|string',
                'password' => ['required', 'confirmed', Rules\Password::defaults()],
                'address' => 'required|string',
            ];

            // Role-specific validation
            $roleRules = [
                'first_name' => 'required|string|max:255',
                'last_name' => 'required|string|max:255',
            ];

            $data = $request->validate(array_merge($baseRules, $roleRules));

            // Create user
            $user = User::create([
                'first_name' => $data['first_name'],
                'last_name' => $data['last_name'],
                'email' => $data['email'],
                'phone' => $data['phone'],
                'country_code' => $data['country_code'],
                'password' => Hash::make($data['password']),
                'role' => $role,
                'address' => $data['address'],
                'api_token' => Str::random(60), // Or use Sanctum tokens
            ]);

            // Create wallet
            $this->createDonorWallet($user); // Or beneficiary wallet

            // Generate access token
            $token = $user->createToken('auth-token')->plainTextToken;

            return response()->json([
                'status' => 'success',
                'message' => 'Registration successful',
                'data' => [
                    'user' => $user,
                    'token' => $token,
                    'wallet' => $user->wallet
                ]
            ], 201);

        } catch (\Illuminate\Validation\ValidationException $e) {
            return response()->json([
                'status' => 'error',
                'message' => 'Validation failed',
                'errors' => $e->errors()
            ], 422);
        }
    }

    public function showRegistrationForm()
    {
        return view('auth.register');
    }

    public function register(Request $request)
    {
        // Validate the common fields first
        $data = $request->validate([
            'email' => 'required|email|unique:users',
            'phone' => 'required|string|unique:users',
            'country_code' => 'required|string',
            'password' => ['required', 'confirmed', Rules\Password::defaults()],
            'role' => 'required|in:beneficiary,vendor,donor',
            'address' => 'required|string',
            'terms' => 'required|accepted',
        ]);

        // Validate conditionally based on role:
        if ($data['role'] === 'vendor') {
            // Vendor fields validation
            $vendorData = $request->validate([
                'business_name' => 'required|string|max:255',
                'admin_name' => 'required|string|max:255',
                'sector' => 'required|string', // Using "sector" for vendor category
            ]);
            $data = array_merge($data, $vendorData);
        } else {
            // For beneficiary and donor, require first and last names
            $individualData = $request->validate([
                'first_name' => 'required|string|max:255',
                'last_name' => 'required|string|max:255',
            ]);
            $data = array_merge($data, $individualData);
        }

        // Create user record conditionally
        if ($data['role'] === 'vendor') {
            $user = User::create([
                'business_name' => $data['business_name'],
                'admin_name' => $data['admin_name'],
                'email' => $data['email'],
                'phone' => $data['phone'],
                'country_code' => $data['country_code'],
                'password' => Hash::make($data['password']),
                'role' => $data['role'],
                'sector' => $data['sector'],
                'address' => $data['address'],
            ]);
        } else {
            $user = User::create([
                'first_name' => $data['first_name'],
                'last_name' => $data['last_name'],
                'email' => $data['email'],
                'phone' => $data['phone'],
                'country_code' => $data['country_code'],
                'password' => Hash::make($data['password']),
                'role' => $data['role'],
                'address' => $data['address'],
            ]);
        }

        // Create wallets based on role
        switch ($data['role']) {
            case 'beneficiary':
                $this->createBeneficiaryWallets($user);
                break;
            case 'vendor':
                $this->createVendorWallet($user, $data['sector']);
                break;
            case 'donor':
                $this->createDonorWallet($user);
                break;
        }

        event(new Registered($user));
        Auth::login($user);

        return redirect()->route('dashboard'); // Adjust as needed
    }

    protected function createBeneficiaryWallets(User $user)
    {
        $wallet = Wallet::create([
            'user_id' => $user->id,
            'wallet_number' => WalletHelper::generateUniqueWalletNumber(),
            'balance' => 0.00,
        ]);

        $categories = ['housing', 'education', 'health', 'food', 'finance', 'utility'];
        foreach ($categories as $category) {
            $subWallet = SubWallet::create([
                'wallet_id' => $wallet->id,
                'category' => $category,
                'balance' => 0.00,
            ]);

            // Create opening transaction for sub-wallet
            $this->createOpeningTransaction($wallet, $subWallet);
        }
    }

    protected function createVendorWallet(User $user, $sector)
    {
        $wallet = Wallet::create([
            'user_id' => $user->id,
            'wallet_number' => WalletHelper::generateUniqueWalletNumber(),
            'balance' => 0.00,
        ]);

        // Create main wallet opening transaction
        $this->createOpeningTransaction($wallet);

        if ($sector) {
            $user->update(['vendor_category' => $sector]);
        }
    }

    protected function createDonorWallet(User $user)
    {
        $wallet = Wallet::create([
            'user_id' => $user->id,
            'wallet_number' => WalletHelper::generateUniqueWalletNumber(),
            'balance' => 0.00,
        ]);

        // Create main wallet opening transaction
        $this->createOpeningTransaction($wallet);
    }

    protected function createOpeningTransaction(Wallet $wallet, SubWallet $subWallet = null)
    {
        Transaction::create([
            'wallet_id' => $wallet->id,
            'sub_wallet_id' => $subWallet?->id,
            'transaction_category' => 'registration',  // Add this line
            'amount' => 0.00,
            'type' => 'opening_balance',
            'reference' => WalletHelper::generateTransactionReference(),
            'narration' => 'Account opening balance',
            'balance_before' => 0.00,
            'balance_after' => 0.00,
            'created_at' => now(),
            'updated_at' => now(),
        ]);
    }
}
