<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Card;
use App\Models\Wallet;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;

class CardController extends Controller
{
    // Existing activation methods...

    // Show card blocking form
    public function showBlockForm()
    {
        return view('cards.block');
    }

    // Process card blocking
    public function processBlock(Request $request)
    {
        $request->validate([
            'card_number' => 'required|exists:cards,card_number',
            'pin' => 'required|digits:4'
        ]);

        try {
            $card = Card::where('card_number', $request->card_number)
                        ->where('user_id', auth()->id())
                        ->firstOrFail();

            if ($card->status === 'blocked') {
                return back()->with('error', 'Card is already blocked.');
            }

            if (!Hash::check($request->pin, $card->hashed_pin)) {
                Log::warning('Invalid PIN attempt for card blocking', ['card' => $card->id]);
                return back()->with('error', 'Invalid PIN.');
            }

            $card->update([
                'status' => 'blocked',
                'blocked_at' => now(),
                'block_reason' => 'user_request'
            ]);

            Log::info('Card blocked successfully', ['card' => $card->id]);

            return redirect()->route('card.block')->with('success', 'Card blocked successfully!');

        } catch (\Exception $e) {
            Log::error('Card blocking failed', ['error' => $e->getMessage()]);
            return back()->with('error', 'Card blocking failed: ' . $e->getMessage());
        }
    }

    // Show stolen card report form
    public function showStolenReportForm()
    {
        return view('cards.report-stolen');
    }

    // Process stolen card report
    public function processStolenReport(Request $request)
    {
        $request->validate([
            'card_number' => 'required|exists:cards,card_number',
            'report_details' => 'required|string|max:500'
        ]);

        try {
            $card = Card::where('card_number', $request->card_number)
                        ->where('user_id', auth()->id())
                        ->firstOrFail();

            if ($card->status === 'stolen') {
                return back()->with('warning', 'This card was already reported stolen.');
            }

            $card->update([
                'status' => 'stolen',
                'reported_stolen_at' => now(),
                'stolen_report_details' => $request->report_details,
                'block_reason' => 'stolen'
            ]);

            Log::critical('Card reported stolen', [
                'card' => $card->id,
                'user' => auth()->id()
            ]);

            // Add any additional logic here (e.g., notify admin, suspend account)

            return redirect()->route('card.report-stolen')->with('success', 'Stolen card reported successfully!');

        } catch (\Exception $e) {
            Log::error('Stolen card report failed', ['error' => $e->getMessage()]);
            return back()->with('error', 'Failed to report card as stolen: ' . $e->getMessage());
        }
    }

    // Add to existing activation method
    public function processActivation(Request $request)
    {
        // Existing activation logic...
    }
}