<?php

// app/Http/Controllers/DonationController.php
namespace App\Http\Controllers;

use App\Models\{Card, SubWallet, Transaction, User, Wallet};
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;

class DonationController extends Controller
{
    // Show donation options
    public function showDonationOptions()
    {
        return view('donations.options');
    }

    // Show card donation form
    public function showCardDonationForm()
    {
        return view('donations.card');
    }

    // Process card donation
    public function processCardDonation(Request $request)
    {
        $request->validate([
            'card_number' => 'required|exists:cards,card_number',
            'amount' => 'required|numeric|min:0.01',
            'sub_wallet' => 'required|exists:sub_wallets,category'
        ]);

        return DB::transaction(function () use ($request) {
            $donor = auth()->user();
            $card = Card::where('card_number', $request->card_number)
                ->with('user.wallet.subWallets')
                ->firstOrFail();

            // Check donor balance
            if ($donor->wallet->balance < $request->amount) {
                return back()->with('error', 'Insufficient balance in your wallet.');
            }

            // Find beneficiary sub-wallet
            $subWallet = $card->user->wallet->subWallets()
                ->where('category', $request->sub_wallet)
                ->firstOrFail();

            // Perform transfer
            $donor->wallet->decrement('balance', $request->amount);
            $subWallet->increment('balance', $request->amount);

            // Record transaction
            Transaction::create([
                'card_id' => $card->id,
                'amount' => $request->amount,
                'type' => 'credit',
                'donation_type' => 'card',
                'description' => "Donation to {$card->user->name}"
            ]);

            return redirect()->route('dashboard')->with('success', 'Donation successful!');
        });
    }

    // Show bulk donation form
    public function showBulkDonationForm(Request $request)
    {
        $query = User::where('role', 'beneficiary');

        // Add filters
        if ($request->has('search')) {
            $query->where('name', 'like', '%' . $request->search . '%');
        }

        $beneficiaries = $query->paginate(10);

        return view('donations.bulk', compact('beneficiaries'));
    }

    // Process bulk donation
    public function processBulkDonation(Request $request)
    {
        $request->validate([
            'beneficiaries' => 'required|array',
            'beneficiaries.*' => 'exists:users,id',
            'amount' => 'required|numeric|min:0.01',
            'sub_wallet' => 'required|exists:sub_wallets,category'
        ]);

        return DB::transaction(function () use ($request) {
            $donor = auth()->user();
            $totalAmount = $request->amount * count($request->beneficiaries);

            // Check donor balance
            if ($donor->wallet->balance < $totalAmount) {
                return back()->with('error', 'Insufficient balance in your wallet.');
            }

            foreach ($request->beneficiaries as $beneficiaryId) {
                $beneficiary = User::findOrFail($beneficiaryId);
                $subWallet = $beneficiary->wallet->subWallets()
                    ->where('category', $request->sub_wallet)
                    ->firstOrFail();

                // Perform transfer
                $donor->wallet->decrement('balance', $request->amount);
                $subWallet->increment('balance', $request->amount);

                // Record transaction
                Transaction::create([
                    'amount' => $request->amount,
                    'type' => 'credit',
                    'donation_type' => 'bulk',
                    'description' => "Bulk donation to {$beneficiary->name}"
                ]);
            }

            return redirect()->route('dashboard')->with('success', 'Bulk donation successful!');
        });
    }
}
