<?php

namespace App\Http\Controllers;

use App\Models\Card;
use App\Models\Transaction;
use App\Models\Wallet;
use App\Services\DonationService;
use App\Helpers\WalletHelper;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use App\Notifications\FundingRequestNotification;
use Illuminate\Http\JsonResponse;
use Illuminate\Support\Facades\Storage;

class DonorController extends Controller
{
    protected $donationService;

    public function __construct(DonationService $donationService)
    {
        $this->donationService = $donationService;
    }

    // Web Interface Methods
    public function showFundingForm()
    {
        return view('donor.fund-wallet');
    }

    public function showDonationForm()
    {
        $categories = ['housing', 'education', 'health', 'food', 'finance', 'utility'];
        return view('donor.donate', compact('categories'));
    }

    public function showBulkDonationForm()
    {
        $categories = ['housing', 'education', 'health', 'food', 'finance', 'utility'];
        return view('donor.bulk-donate', compact('categories'));
    }

    // Combined Web + API Methods
    public function fundWallet(Request $request)
    {
        $isApiRequest = $request->is('api/*');

        $validationRules = [
            'amount' => 'required|numeric|min:1',
        ];

        if ($isApiRequest) {
            $validationRules['payment_proof'] = 'required|string';
            $validationRules['payment_method'] = 'required|in:bank_transfer,card,paypal';
        } else {
            $validationRules['payment_proof'] = 'required|file|mimes:jpg,png,pdf|max:2048';
        }

        $validated = $request->validate($validationRules);

        try {
            $user = Auth::user();
            $wallet = $user->wallet;

            // Handle payment proof
            if ($isApiRequest) {
                $imageData = base64_decode($validated['payment_proof']);
                $fileName = 'proof_'.time().'.png';
                Storage::put("payment-proofs/{$fileName}", $imageData);
                $proofPath = "payment-proofs/{$fileName}";
            } else {
                $proofPath = $request->file('payment_proof')->store('payment-proofs');
            }

            $transaction = Transaction::create([
                'wallet_id' => $wallet->id,
                'amount' => $validated['amount'],
                'type' => 'credit',
                'reference' => WalletHelper::generateTransactionReference(),
                'transaction_category' => 'wallet_funding',
                'description' => $isApiRequest 
                    ? 'Wallet funding via '.$validated['payment_method']
                    : 'Wallet funding - pending approval',
                'balance_before' => $wallet->balance,
                'balance_after' => $wallet->balance,
                'status' => 'pending',
                'meta' => $isApiRequest ? [
                    'payment_method' => $validated['payment_method'],
                    'proof_url' => Storage::url($proofPath)
                ] : null
            ]);

            $user->notify(new FundingRequestNotification($transaction));

            if ($isApiRequest) {
                return response()->json([
                    'status' => 'success',
                    'message' => 'Funding request submitted',
                    'data' => [
                        'reference' => $transaction->reference,
                        'amount' => $transaction->amount,
                        'status' => $transaction->status
                    ]
                ], 202);
            }

            return redirect()->route('donor.dashboard')->with('success', 'Funding request submitted for approval');

        } catch (\Exception $e) {
            if ($isApiRequest) {
                return response()->json([
                    'status' => 'error',
                    'message' => 'Funding failed: '.$e->getMessage()
                ], 500);
            }

            return redirect()->back()
                ->with('error', $e->getMessage())
                ->withInput();
        }
    }

    public function processDonation(Request $request)
    {
        $isApiRequest = $request->is('api/*');

        $validated = $request->validate([
            'amount' => 'required|numeric|min:1',
            'category' => 'required|in:housing,education,health,food,finance,utility',
            'card_numbers' => 'required|array',
            'card_numbers.*' => 'exists:cards,card_number'
        ]);

        DB::beginTransaction();
        try {
            $donor = Auth::user();
            $cards = Card::whereIn('card_number', $validated['card_numbers'])
                ->with('wallet.subWallets')
                ->get();

            $result = $this->donationService->processDonation(
                $donor->wallet,
                $validated['amount'],
                $cards,
                $validated['category']
            );

            DB::commit();

            if ($isApiRequest) {
                return response()->json([
                    'status' => 'success',
                    'message' => 'Donation processed',
                    'data' => [
                        'total_recipients' => count($cards),
                        'total_amount' => $validated['amount'] * count($cards),
                        'transactions' => $result
                    ]
                ]);
            }

            return redirect()->route('donor.dashboard')
                ->with('success', 'Donation processed successfully');

        } catch (\Exception $e) {
            DB::rollBack();
            
            if ($isApiRequest) {
                return response()->json([
                    'status' => 'error',
                    'message' => 'Donation failed: '.$e->getMessage()
                ], 400);
            }

            return redirect()->back()
                ->with('error', $e->getMessage())
                ->withInput();
        }
    }

    public function processBulkDonation(Request $request)
    {
        $isApiRequest = $request->is('api/*');

        $validationRules = [
            'amount' => 'required|numeric|min:1',
            'category' => 'required|in:housing,education,health,food,finance,utility',
        ];

        if ($isApiRequest) {
            $validationRules['card_numbers'] = 'required|string';
        } else {
            $validationRules['csv_file'] = 'required|file|mimes:csv,txt';
        }

        $validated = $request->validate($validationRules);

        try {
            // Process input
            if ($isApiRequest) {
                $cardNumbers = array_unique(
                    explode(',', preg_replace('/\s+/', '', $validated['card_numbers']))
                );
            } else {
                $cardNumbers = $this->processCSV($request->file('csv_file')->getRealPath());
            }

            $donor = Auth::user();
            $cards = Card::whereIn('card_number', $cardNumbers)
                ->with('wallet.subWallets')
                ->get();

            $result = $this->donationService->processDonation(
                $donor->wallet,
                $validated['amount'],
                $cards,
                $validated['category']
            );

            if ($isApiRequest) {
                return response()->json([
                    'status' => 'success',
                    'message' => 'Bulk donation processed',
                    'data' => [
                        'cards_processed' => count($result),
                        'cards_failed' => count($cardNumbers) - count($result),
                        'transactions' => $result
                    ]
                ]);
            }

            return redirect()->route('donor.dashboard')
                ->with('success', 'Bulk donation processed successfully');

        } catch (\Exception $e) {
            if ($isApiRequest) {
                return response()->json([
                    'status' => 'error',
                    'message' => 'Bulk donation failed: '.$e->getMessage()
                ], 400);
            }

            return redirect()->back()
                ->with('error', $e->getMessage())
                ->withInput();
        }
    }

    // API-Specific Methods
    public function getTransactions(Request $request): JsonResponse
    {
        $transactions = $request->user()->wallet->transactions()
            ->latest()
            ->paginate(10);

        return response()->json([
            'status' => 'success',
            'data' => $transactions
        ]);
    }

    // Shared Helper Methods
    private function processCSV($filePath)
    {
        $cardNumbers = [];
        if (($handle = fopen($filePath, 'r')) !== false) {
            while (($data = fgetcsv($handle))) {
                $cardNumbers[] = $data[0];
            }
            fclose($handle);
        }
        return array_unique($cardNumbers);
    }
}