<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Card;
use App\Models\SubWallet;
use App\Models\Transaction;
use App\Models\Vendor;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\DB;

class TransactionController extends Controller
{
    /**
     * Display the transaction form.
     */
    public function showTransactionForm()
    {
        return view('transactions.create', [
            'vendorCategory' => auth()->user()->vendor->category
        ]);
    }

    public function index()
    {
        $transactions = auth()->user()->wallet->transactions()
            ->with('subWallet')
            ->orderBy('created_at', 'desc')
            ->paginate(10);

        return view('transactions.index', compact('transactions'));
    }

    /**
     * Process the submitted transaction form.
     */
    public function processTransaction(Request $request)
    {
        $validated = $request->validate([
            'card_number' => 'required|string|size:11', // Assuming 11-digit card numbers
            'amount' => 'required|numeric|min:0.01|decimal:0,2',
            'description' => 'required|string|max:255',
            'pin' => 'required|digits:4'
        ]);

        return DB::transaction(function () use ($validated) {
            // 1. Retrieve and validate the card with wallet relationship
            $card = Card::with('wallet.subWallets')
                ->where('card_number', $validated['card_number'])
                ->where('status', 'active')
                ->firstOrFail();

            // 2. Get authenticated vendor
            $vendor = auth()->user()->vendor;
            $vendorCategory = strtolower($vendor->category);

            // 3. Find matching subwallet
            $subWallet = $card->wallet->subWallets
                ->firstWhere('category', $vendorCategory);

            // 4. Check funds
            if (!$subWallet) {
                return back()->with('error', 'No matching subwallet found for this category.');
            }

            if ($subWallet->balance < $validated['amount']) {
                return back()->with('error', 'Insufficient funds in ' . ucfirst($vendorCategory) . ' subwallet.');
            }

            // 5. Verify PIN
            if (!Hash::check($validated['pin'], $card->hashed_pin)) {
                return back()->with('error', 'Invalid PIN. Please try again.');
            }

            // 6. Process transaction
            $subWallet->decrement('balance', $validated['amount']);

            Transaction::create([
                'card_id' => $card->id,
                'vendor_id' => $vendor->id,
                'amount' => $validated['amount'],
                'type' => 'debit',
                'category' => $vendorCategory,
                'description' => $validated['description'],
            ]);

            return redirect()
                ->route('transaction.form')
                ->with('success', '₦' . number_format($validated['amount'], 2) . ' successfully deducted from ' . ucfirst($vendorCategory) . ' subwallet.');
        });
    }
}