<?php

namespace App\Http\Controllers;

use App\Models\Card;
use App\Models\SubWallet;
use App\Models\Transaction;
use App\Models\Payout;
use App\Models\Wallet;
use App\Models\User;
use App\Models\Kyc;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\DB;
// Use your custom exceptions if defined, or comment these out if not available:
// use App\Exceptions\CardLockedException;
// use App\Exceptions\InvalidPinException;

class VendorController extends Controller
{
    /**
     * Show the vendor settings page.
     */
    public function settings()
    {
        $user = Auth::user();
        $kyc = Kyc::where('user_id', $user->id)->first();

        if (!$user) {
            return redirect()->route('login')->with('error', 'You must be logged in to access this page.');
        }

        return view('vendors.settings', compact('user', 'kyc'));
    }

    /**
     * Update vendor profile.
     */
    public function updateProfile(Request $request)
    {
        $user = Auth::user();
        $validated = $request->validate([
            'business_name' => 'required|string|max:255',
            'admin_name' => 'required|string|max:255',
            'phone' => 'required|string|max:20',
            'email' => 'required|email',
            'sector' => 'nullable|string',
            'address' => 'required|string',
        ]);

        $user->update($validated);

        return redirect()->back()->with('success', 'Profile updated successfully.');
    }

    /**
     * Update vendor KYB.
     */
    public function updateKYB(Request $request)
    {
        $user = Auth::user();
        $kyc = Kyc::firstOrCreate(['user_id' => $user->id]);

        $validated = $request->validate([
            'full_name' => 'required|string|max:255',
            'date_of_birth' => 'required|date',
            'nationality' => 'required|string|max:255',
            'phone_number' => 'required|string|max:20',
            'id_type' => 'required|string',
            'id_number' => 'required|string',
            'business_registration_number' => 'nullable|string|max:255',
            'tax_number' => 'nullable|string|max:255',
            'bank_account_name' => 'nullable|string|max:255',
            'bvn' => 'nullable|string|max:11',
        ]);

        $kyc->update($validated);

        return redirect()->back()->with('success', 'KYB updated successfully.');
    }

    /**
     * Display the vendor dashboard with dynamic data.
     */
    public function dashboard()
    {
        $vendor = Auth::user();
        $wallet = $vendor->wallet;

        if (!$wallet) {
            return redirect()->back()->with('error', 'Wallet not found for this vendor.');
        }

        // Convert vendor categories (comma separated) into an array.
        $vendorCategories = explode(',', $vendor->vendor_category);

        // Fetch transaction totals per category for this wallet.
        $categoryData = Transaction::whereIn('transaction_category', $vendorCategories)
            ->where('wallet_id', $wallet->id)
            ->selectRaw('transaction_category, SUM(amount) as total')
            ->groupBy('transaction_category')
            ->get();

        $categoryLabels = $categoryData->pluck('transaction_category')->toArray();
        $categoryTotals = $categoryData->pluck('total')->toArray();

        if (empty($categoryLabels)) {
            $categoryLabels = $vendorCategories;
            $categoryTotals = array_fill(0, count($vendorCategories), rand(5000, 15000));
        }

        // Fetch monthly transaction totals.
        $transactionData = Transaction::where('wallet_id', $wallet->id)
            ->selectRaw('MONTH(created_at) as month, SUM(amount) as total')
            ->groupBy('month')
            ->orderBy('month')
            ->get();

        $transactionMonths = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
        $transactionTotals = array_fill(0, 12, 0);
        foreach ($transactionData as $data) {
            $index = $data->month - 1;
            $transactionTotals[$index] = $data->total;
        }

        if (array_sum($transactionTotals) == 0) {
            $transactionTotals = array_map(fn() => rand(5000, 20000), range(1, 12));
        }

        // Recent transactions (paginated).
        $recentTransactions = Transaction::where('wallet_id', $wallet->id)
            ->orderBy('created_at', 'desc')
            ->paginate(10);

        if ($recentTransactions->isEmpty()) {
            $sampleData = collect([
                (object) [
                    'transaction_category' => 'Education',
                    'description' => 'School Books',
                    'type' => 'debit',
                    'amount' => 15000,
                    'created_at' => now()->subDays(1)
                ],
                (object) [
                    'transaction_category' => 'Education',
                    'description' => 'School Uniform',
                    'type' => 'debit',
                    'amount' => 7500,
                    'created_at' => now()->subDays(2)
                ]
            ]);

            $perPage = 10;
            $page = request()->get('page', 1);
            $offset = ($page - 1) * $perPage;
            $recentTransactions = new LengthAwarePaginator(
                $sampleData->slice($offset, $perPage)->values(),
                $sampleData->count(),
                $perPage,
                $page,
                ['path' => request()->url()]
            );
        }

        // KYC Warning if not approved.
        $kycStatus = optional($vendor->kyc)->status ?? 'pending';
        $kycWarning = ($kycStatus !== 'approved') ? "Your KYB verification is incomplete. Please complete your KYB to start transactions." : null;

        // Payout summary.
        $pendingPayments = Payout::where('wallet_id', $wallet->id)
            ->where('status', 'pending')
            ->sum('amount');
        $totalPayouts = Payout::where('wallet_id', $wallet->id)
            ->where('status', 'completed')
            ->sum('amount');

        // Wallet details.
        $walletnumber = $wallet->wallet_number ?? 'Not Available';
        $walletbalance = $wallet->balance ?? 0;

        // Recent activities.
        $recentActivities = Transaction::where('wallet_id', $wallet->id)
            ->orderBy('created_at', 'desc')
            ->take(5)
            ->get(['description', 'amount', 'created_at']);

        return view('vendors.dashboard', [
            'walletnumber' => $walletnumber,
            'walletbalance' => $walletbalance,
            'pendingPayments' => $pendingPayments,
            'totalPayouts' => $totalPayouts,
            'transactionMonths' => $transactionMonths,
            'transactionTotals' => $transactionTotals,
            'categoryLabels' => $categoryLabels,
            'categoryTotals' => $categoryTotals,
            'recentActivities' => $recentActivities,
            'recentTransactions' => $recentTransactions,
            'kycWarning' => $kycWarning,
        ]);
    }

    /**
     * Process a transaction.
     */
    public function processTransaction(Request $request)
    {
        \Log::info('Transaction attempt started', $request->all());

        $validated = $request->validate([
            'card_number' => 'required|exists:cards,card_number',
            'amount' => 'required|numeric|min:1',
            'category' => 'required|in:housing,education,health,food,finance,utility',
            'description' => 'required|string|max:70',
            'pin' => 'required|digits:4'
        ]);

        // Check vendor's allowed categories
        if (!in_array($validated['category'], explode(',', auth()->user()->vendor_category))) {
            \Log::warning('Unauthorized category', [
                'vendor_categories' => auth()->user()->vendor_category,
                'attempted' => $validated['category']
            ]);
            return back()->with('error', 'Unauthorized transaction category.');
        }

        DB::beginTransaction();
        \Log::info('Transaction started');

        try {
            $card = Card::where('card_number', $validated['card_number'])
                ->with(['wallet.subWallets', 'user'])
                ->firstOrFail();
            \Log::info('Card found', ['card' => $card->id]);

            // Check PIN attempts
            if ($card->pin_attempts >= 3) {
                \Log::warning('Card locked', ['card' => $card->id]);
                throw new \Exception('Card is locked due to multiple invalid PIN attempts.');
            }

            if (!Hash::check($validated['pin'], $card->hashed_pin)) {
                $card->increment('pin_attempts');
                \Log::warning('Invalid PIN attempt', ['card' => $card->id, 'attempts' => $card->pin_attempts]);
                throw new \Exception('Invalid PIN. You have ' . (3 - $card->pin_attempts) . ' attempts remaining.');
            }

            $card->update(['pin_attempts' => 0]);
            \Log::info('PIN verified', ['card' => $card->id]);

            // --- MOVED KYC CHECK HERE ---
            $user = auth()->user();
            \Log::info('Initiating KYC verification check', ['user_id' => $user->id]);
            $kyc = $user->kyc;
            $kycStatus = $kyc ? $kyc->status : 'not-submitted';

            if (!$kyc || $kyc->status !== 'approved') {
                \Log::warning('KYC verification failed', [
                    'user_id' => $user->id,
                    'kyc_status' => $kycStatus,
                    'required_status' => 'approved'
                ]);
                throw new \Exception("KYC verification required (Status: " . strtoupper($kycStatus) . ")");
            }
            \Log::info('KYC verification passed', ['user_id' => $user->id]);

            // Get beneficiary's subwallet
            $subWallet = $card->wallet->subWallets()
                ->where('category', $validated['category'])
                ->firstOrFail();
            \Log::info('Sub-wallet found', ['sub_wallet' => $subWallet->id]);

            // Balance checks
            \Log::info('Sub-wallet validation', [
                'current_balance' => $subWallet->balance,
                'requested_amount' => $validated['amount']
            ]);

            if ($subWallet->balance < $validated['amount']) {
                \Log::warning('Insufficient sub-wallet balance', [
                    'deficit' => $validated['amount'] - $subWallet->balance
                ]);
                throw new \Exception('Insufficient balance in sub-wallet.');
            }

            // Deduct funds
            $reference = 'TX-' . Str::random(10);
            $balanceBefore = $subWallet->balance;

            \Log::info('Initiating deduction', [
                'amount' => $validated['amount'],
                'reference' => $reference
            ]);

            $subWallet->decrement('balance', $validated['amount']);
            \Log::info('Deduction successful', [
                'new_balance' => $subWallet->balance,
                'balance_change' => $balanceBefore - $subWallet->balance
            ]);

            // Vendor wallet handling
            $vendorWallet = $user->wallet;
            if (!$vendorWallet) {
                \Log::emergency('Vendor wallet missing', ['user_id' => $user->id]);
                throw new \Exception('Wallet not found');
            }

            // Create transactions
            Transaction::create([/* beneficiary transaction */]);
            Transaction::create([/* vendor transaction */]);

            DB::commit();
            \Log::info('Transaction completed successfully');
            return redirect()->route('vendor.dashboard')->with('success', 'Transaction completed successfully!');

        } catch (\Exception $e) {
            DB::rollBack();
            \Log::error('Transaction failed', ['error' => $e->getMessage()]);
            return redirect()->back()->withInput()->with('error', $e->getMessage());
        }
    }

    public function showKycPage()
    {
        return view('vendors.kyc');
    }

    public function showTransactionForm()
    {
        $categories = explode(',', auth()->user()->vendor_category);
        return view('vendor.transact', compact('categories'));
    }

    public function submitKyc(Request $request)
    {
        $request->validate([
            'document' => 'required|file|mimes:jpg,jpeg,png,pdf|max:2048',
        ]);

        $vendor = Auth::user();
        $path = $request->file('document')->store('kyc-documents');

        $vendor->kyc()->updateOrCreate(
            ['user_id' => $vendor->id],
            ['status' => 'pending', 'document' => $path]
        );

        return redirect()->route('vendor.dashboard')->with('success', 'KYC submitted successfully. Awaiting approval.');
    }

    public function showRedemptionForm()
    {
        return view('vendors.redeem');
    }

    public function redeem(Request $request)
    {
        $request->validate([
            'card_number' => 'required|exists:cards,card_number',
            'amount' => 'required|numeric|min:1',
            'transaction_category' => 'required|string',
        ]);

        $vendor = Auth::user();
        $wallet = $vendor->wallet;
        $card = Card::where('card_number', $request->card_number)->first();

        if (!$wallet) {
            return redirect()->back()->with('error', 'Wallet not found');
        }

        $subWallet = $card->wallet->subWallets()
            ->where('category', $request->transaction_category)
            ->first();

        if (!$subWallet || $subWallet->balance < $request->amount) {
            return redirect()->back()->with('error', 'Insufficient balance or invalid category.');
        }

        if (!$vendor->kyc || $vendor->kyc->status !== 'approved') {
            return redirect()->back()->with('error', 'You must complete KYB verification to transact.');
        }

        $subWallet->decrement('balance', $request->amount);
        Transaction::create([
            'wallet_id' => $wallet->id,
            'card_id' => $card->id,
            'amount' => $request->amount,
            'type' => 'debit',
            'transaction_category' => $request->transaction_category,
            'description' => 'Redemption by ' . $vendor->business_name,
        ]);

        return redirect()->back()->with('success', 'Redemption successful!');
    }

    public function showVerifyPinForm(Request $request)
    {
        if (!$request->session()->has('transaction_data')) {
            return redirect()->route('vendor.transact.form')->with('error', 'Transaction session expired or invalid');
        }

        return view('vendor.verify-pin');
    }

    public function verifyPin(Request $request)
    {
        $transactionData = $request->session()->get('transaction_data');

        if (!$transactionData) {
            return redirect()->route('vendor.transact.form')->with('error', 'Transaction session expired');
        }

        $request->validate(['pin' => 'required|digits:4']);

        try {
            DB::beginTransaction();

            $card = Card::where('card_number', $transactionData['card_number'])
                ->with(['wallet.subWallets', 'user'])
                ->firstOrFail();

            if ($card->pin_attempts >= 3) {
                throw new \Exception('Card is locked due to multiple invalid PIN attempts.');
            }

            if (!Hash::check($request->pin, $card->hashed_pin)) {
                $card->increment('pin_attempts');
                throw new \Exception('Invalid PIN. You have ' . (3 - $card->pin_attempts) . ' attempts remaining.');
            }

            $card->update(['pin_attempts' => 0]);

            $subWallet = $card->wallet->subWallets()
                ->where('category', $transactionData['category'])
                ->firstOrFail();

            if ($subWallet->balance < $transactionData['amount']) {
                throw new \Exception('Insufficient balance in sub-wallet');
            }

            $subWallet->decrement('balance', $transactionData['amount']);

            Transaction::create([
                'wallet_id' => $card->wallet_id,
                'sub_wallet_id' => $subWallet->id,
                'amount' => $transactionData['amount'],
                'type' => 'debit',
                'reference' => 'TX-' . Str::random(10), // Generate a unique reference
                'transaction_category' => $transactionData['category'],
                'description' => $transactionData['description'],
                'balance_before' => $subWallet->balance + $transactionData['amount'],
                'balance_after' => $subWallet->balance,
                'created_at' => now(),
                'updated_at' => now()
            ]);

            $vendorWallet = Auth::user()->wallet;
            $vendorWallet->increment('balance', $transactionData['amount']);

            Transaction::create([
                'wallet_id' => $vendorWallet->id,
                'amount' => $transactionData['amount'],
                'type' => 'credit',
                'reference' => 'VENDOR-' . Str::random(10),
                'transaction_category' => $transactionData['category'],
                'description' => 'Payment from ' . $card->card_number,
                'balance_before' => $vendorWallet->balance - $transactionData['amount'],
                'balance_after' => $vendorWallet->balance,
                'created_at' => now(),
                'updated_at' => now()
            ]);

            DB::commit();

            $request->session()->forget('transaction_data');

            return redirect()->route('vendor.dashboard')->with('success', 'Transaction completed successfully');

        } catch (\Exception $e) {
            DB::rollBack();
            return redirect()->back()->withInput()->with('error', $e->getMessage());
        }
    }
}
