<?php

namespace App\Http\Controllers;

use App\Models\Card;
use App\Models\SubWallet;
use App\Models\Transaction;
use App\Models\Payout;
use App\Models\Wallet;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Support\Collection;
use App\Models\User;
use App\Models\Kyc;
use App\Exceptions\CardLockedException;
use App\Exceptions\InvalidPinException;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\DB;


class VendorController2 extends Controller
{
    public function settings()
    {
        $user = Auth::user();
        $kyc = Kyc::where('user_id', $user->id)->first();

        if (!$user) {
            return redirect()->route('login')->with('error', 'You must be logged in to access this page.');
        }

        return view('vendors.settings', compact('user', 'kyc'));
    }


    public function updateProfile(Request $request)
    {
        $user = Auth::user();
        $request->validate([
            'business_name' => 'required|string|max:255',
            'admin_name' => 'required|string|max:255',
            'phone' => 'required|string|max:20',
            'email' => 'required|email',
            'sector' => 'nullable|string',
            'address' => 'required|string',
        ]);

        $user->update($request->all());

        return redirect()->back()->with('success', 'Profile updated successfully');
    }

    public function updateKYB(Request $request)
    {
        $user = Auth::user();
        $kyc = Kyc::firstOrCreate(['user_id' => $user->id]);

        $request->validate([
            'full_name' => 'required|string|max:255',
            'date_of_birth' => 'required|date',
            'nationality' => 'required|string|max:255',
            'phone_number' => 'required|string|max:20',
            'id_type' => 'required|string',
            'id_number' => 'required|string',
            'business_registration_number' => 'nullable|string|max:255',
            'tax_number' => 'nullable|string|max:255',
            'bank_account_name' => 'nullable|string|max:255',
            'bvn' => 'nullable|string|max:11',
        ]);

        $kyc->update($request->all());

        return redirect()->back()->with('success', 'KYB updated successfully');
    }

    public function dashboard()
    {
        $vendor = Auth::user();
        $wallet = $vendor->wallet;

        if (!$wallet) {
            return redirect()->back()->with('error', 'Wallet not found for this vendor');
        }

        // Convert vendor categories into an array
        $vendorCategories = explode(',', $vendor->vendor_category);

        // Fetch transaction totals per category for the vendor
        $categoryData = Transaction::whereIn('transaction_category', $vendorCategories)
            ->where('wallet_id', $wallet->id)
            ->selectRaw('transaction_category, SUM(amount) as total')
            ->groupBy('transaction_category')
            ->get();

        $categoryLabels = $categoryData->pluck('transaction_category')->toArray();
        $categoryTotals = $categoryData->pluck('total')->toArray();

        // Handle empty chart case
        if (empty($categoryLabels)) {
            $categoryLabels = $vendorCategories;
            $categoryTotals = array_fill(0, count($vendorCategories), rand(5000, 15000));
        }

        // Fetch transaction data per month
        $transactionData = Transaction::where('wallet_id', $wallet->id)
            ->selectRaw('MONTH(created_at) as month, SUM(amount) as total')
            ->groupBy('month')
            ->orderBy('month')
            ->get();

        $transactionMonths = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
        $transactionTotals = array_fill(0, 12, 0);
        foreach ($transactionData as $data) {
            $index = $data->month - 1;
            $transactionTotals[$index] = $data->total;
        }

        if (array_sum($transactionTotals) == 0) {
            $transactionTotals = array_map(fn() => rand(5000, 20000), range(1, 12));
        }

        // Fetch paginated transactions
        $recentTransactions = Transaction::where('wallet_id', $wallet->id)
            ->orderBy('created_at', 'desc')
            ->paginate(10);

        // Handle empty transactions
        if ($recentTransactions->isEmpty()) {
            $sampleData = collect([
                [
                    'wallet_number' => '1234567890',
                    'name' => 'John Doe',
                    'category' => 'Education',
                    'details' => 'School Books',
                    'amount' => '₦15,000',
                    'date' => now()->subDays(1)->format('d/m/Y'),
                ],
                [
                    'wallet_number' => '0987654321',
                    'name' => 'Jane Smith',
                    'category' => 'Education',
                    'details' => 'School Uniform',
                    'amount' => '₦7,500',
                    'date' => now()->subDays(2)->format('d/m/Y'),
                ],
            ]);

            $perPage = 10;
            $page = request()->get('page', 1);
            $offset = ($page - 1) * $perPage;
            $recentTransactions = new LengthAwarePaginator(
                $sampleData->slice($offset, $perPage)->values(),
                $sampleData->count(),
                $perPage,
                $page,
                ['path' => request()->url()]
            );
        }

        // KYC status
        $kycStatus = optional($vendor->kyc)->status ?? 'pending';
        $kycWarning = ($kycStatus !== 'approved') ? "Your KYB verification is incomplete. Please complete your KYB to start transactions." : null;

        // Payout summary
        $pendingPayments = Payout::where('wallet_id', $wallet->id)
            ->where('status', 'pending')
            ->sum('amount');

        $totalPayouts = Payout::where('wallet_id', $wallet->id)
            ->where('status', 'completed')
            ->sum('amount');

        // Wallet details
        $walletnumber = $wallet->wallet_number ?? 'Not Available';
        $walletbalance = $wallet->balance ?? 0;

        // Recent activities
        $recentActivities = Transaction::where('wallet_id', $wallet->id)
            ->orderBy('created_at', 'desc')
            ->take(5)
            ->get(['description', 'amount', 'created_at']);

        return view('vendors.dashboard', [
            'walletnumber' => $walletnumber,
            'walletbalance' => $walletbalance,
            'pendingPayments' => $pendingPayments,
            'totalPayouts' => $totalPayouts,
            'transactionMonths' => $transactionMonths,
            'transactionTotals' => $transactionTotals,
            'categoryLabels' => $categoryLabels,
            'categoryTotals' => $categoryTotals,
            'recentActivities' => $recentActivities,
            'recentTransactions' => $recentTransactions,
            'kycWarning' => $kycWarning,
        ]);
    }

    public function processTransaction(Request $request)
    {
        \Log::info('Transaction attempt started', $request->all());

        $validated = $request->validate([
            'card_number' => 'required|exists:cards,card_number',
            'amount' => 'required|numeric|min:1',
            'category' => 'required|in:housing,education,health,food,finance,utility',
            'description' => 'required|string|max:70',
            'pin' => 'required|digits:4'
        ]);

        // Vendor authorization
        if (!in_array($request->category, explode(',', auth()->user()->vendor_category))) {
            \Log::warning('Unauthorized category', [
                'vendor_categories' => auth()->user()->vendor_category,
                'attempted' => $request->category
            ]);
            return back()->with('error', 'Unauthorized transaction category');
        }

        DB::beginTransaction();
        \Log::info('Transaction started');

        try {
            $card = Card::where('card_number', $request->card_number)
                ->with(['wallet.subWallets', 'user'])
                ->firstOrFail();
            \Log::info('Card found', ['card' => $card->id]);

            // PIN handling
            if ($card->pin_attempts >= 3) {
                \Log::warning('Card locked', ['card' => $card->id]);
                throw new CardLockedException();
            }

            if (!Hash::check($request->pin, $card->hashed_pin)) {
                $card->increment('pin_attempts');
                \Log::warning('Invalid PIN attempt', [
                    'card' => $card->id,
                    'attempts' => $card->pin_attempts
                ]);
                throw new InvalidPinException(3 - $card->pin_attempts);
            }

            $card->update(['pin_attempts' => 0]);
            \Log::info('PIN verified', ['card' => $card->id]);

            // Sub-wallet handling
            $subWallet = $card->wallet->subWallets()
                ->where('category', $request->category)
                ->firstOrFail();
            \Log::info('Sub-wallet found', [
                'sub_wallet' => $subWallet->id,
                'balance' => $subWallet->balance
            ]);

            if ($subWallet->balance < $request->amount) {
                \Log::warning('Insufficient balance', [
                    'balance' => $subWallet->balance,
                    'amount' => $request->amount
                ]);
                throw new \Exception('Insufficient balance in sub-wallet');
            }

            // Transaction records
            $reference = 'TX-' . Str::random(10);
            $balanceBefore = $subWallet->balance;

            // Deduct from beneficiary
            $subWallet->decrement('balance', $request->amount);
            Transaction::create([/*...*/]);
            \Log::info('Beneficiary transaction created', ['reference' => $reference]);

            // Credit vendor
            $vendorWallet = auth()->user()->wallet;
            $vendorReference = 'VENDOR-' . Str::random(10);
            $vendorWallet->increment('balance', $request->amount);
            Transaction::create([/*...*/]);
            \Log::info('Vendor transaction created', ['reference' => $vendorReference]);

            DB::commit();
            \Log::info('Transaction completed successfully');

            return redirect()->route('vendor.dashboard')
                ->with('success', 'Transaction completed successfully');

        } catch (\Exception $e) {
            DB::rollBack();
            \Log::error('Transaction failed', [
                'error' => $e->getMessage(),
                'trace' => $e->getTraceAsString()
            ]);
            return back()->withInput()->with('error', $e->getMessage());
        }
    }

    public function showKycPage()
    {
        return view('vendors.kyc');
    }



    public function showTransactionForm()
    {
        $categories = explode(',', auth()->user()->vendor_category);
        return view('vendor.transact', compact('categories'));
    }

    public function submitKyc(Request $request)
    {
        $request->validate([
            'document' => 'required|file|mimes:jpg,jpeg,png,pdf|max:2048',
        ]);

        $vendor = Auth::user();
        $path = $request->file('document')->store('kyc-documents');

        $vendor->kyc()->updateOrCreate(
            ['user_id' => $vendor->id],
            ['status' => 'pending', 'document' => $path]
        );

        return redirect()->route('vendors.dashboard')->with('success', 'KYC submitted successfully. Awaiting approval.');
    }

    public function showRedemptionForm()
    {
        return view('vendors.redeem');
    }

    public function redeem(Request $request)
    {
        $request->validate([
            'card_number' => 'required|exists:cards,card_number',
            'amount' => 'required|numeric|min:1',
            'transaction_category' => 'required|string',
        ]);

        $vendor = Auth::user();
        $wallet = $vendor->wallet;
        $card = Card::where('card_number', $request->card_number)->first();

        if (!$wallet) {
            return redirect()->back()->with('error', 'Wallet not found');
        }

        $subWallet = $card->wallet->subWallets()
            ->where('category', $request->transaction_category)
            ->first();

        if (!$subWallet || $subWallet->balance < $request->amount) {
            return redirect()->back()->with('error', 'Insufficient balance or invalid category.');
        }

        if (!$vendor->kyc || $vendor->kyc->status !== 'approved') {
            return redirect()->back()->with('error', 'You must complete KYB verification to transact.');
        }

        $subWallet->balance -= $request->amount;
        $subWallet->save();

        Transaction::create([
            'wallet_id' => $wallet->id,
            'card_id' => $card->id,
            'amount' => $request->amount,
            'type' => 'debit',
            'transaction_category' => $request->transaction_category,
            'description' => 'Redemption by ' . $vendor->business_name,
        ]);

        return redirect()->back()->with('success', 'Redemption successful!');
    }

    public function showVerifyPinForm(Request $request)
    {
        // Check if we have transaction data in session
        if (!$request->session()->has('transaction_data')) {
            return redirect()->route('vendor.transact.form')
                ->with('error', 'Transaction session expired or invalid');
        }

        return view('vendor.verify-pin');
    }

    public function verifyPin(Request $request)
    {
        $transactionData = $request->session()->get('transaction_data');

        if (!$transactionData) {
            return redirect()->route('vendor.transact.form')->with('error', 'Transaction session expired');
        }

        $request->validate(['pin' => 'required|digits:4']);

        try {
            DB::beginTransaction();

            $card = Card::where('card_number', $transactionData['card_number'])
                ->with(['wallet.subWallets', 'user'])
                ->firstOrFail();

            // Check PIN attempts
            if ($card->pin_attempts >= 3) {
                throw new CardLockedException();
            }

            // Verify PIN
            if (!Hash::check($request->pin, $card->hashed_pin)) {
                $card->increment('pin_attempts');
                throw new InvalidPinException(3 - $card->pin_attempts);
            }

            // Reset PIN attempts on success
            $card->update(['pin_attempts' => 0]);

            // Get the sub-wallet
            $subWallet = $card->wallet->subWallets()
                ->where('category', $transactionData['category'])
                ->firstOrFail();

            // Check balance
            if ($subWallet->balance < $transactionData['amount']) {
                throw new \Exception('Insufficient balance in sub-wallet');
            }

            // Deduct from beneficiary
            $subWallet->decrement('balance', $transactionData['amount']);

            // Create beneficiary transaction
            Transaction::create([
                'wallet_id' => $card->wallet_id,
                'sub_wallet_id' => $subWallet->id,
                'amount' => $transactionData['amount'],
                'type' => 'debit',
                'reference' => $transactionData['reference'],
                'transaction_category' => $transactionData['category'],
                'description' => $transactionData['description'],
                'balance_before' => $subWallet->balance + $transactionData['amount'],
                'balance_after' => $subWallet->balance,
                'created_at' => now(),
                'updated_at' => now()
            ]);

            // Add to vendor's wallet
            $vendorWallet = Auth::user()->wallet;
            $vendorWallet->increment('balance', $transactionData['amount']);

            // Create vendor transaction
            Transaction::create([
                'wallet_id' => $vendorWallet->id,
                'amount' => $transactionData['amount'],
                'type' => 'credit',
                'reference' => 'VENDOR-' . Str::random(10),
                'transaction_category' => $transactionData['category'],
                'description' => 'Payment from ' . $card->card_number,
                'balance_before' => $vendorWallet->balance - $transactionData['amount'],
                'balance_after' => $vendorWallet->balance,
                'created_at' => now(),
                'updated_at' => now()
            ]);

            DB::commit();

            // Clear session data
            $request->session()->forget('transaction_data');

            return redirect()->route('vendor.dashboard')
                ->with('success', 'Transaction completed successfully');

        } catch (\Exception $e) {
            DB::rollBack();
            return redirect()->back()
                ->withInput()
                ->with('error', $e->getMessage());
        }
    }


}