<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Hash;

class Card extends Model
{
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'card_number',
        'pin', // This will be hashed automatically via mutator
        'hashed_pin',
        'pin_attempts',
        'last_pin_attempt',
        'user_id',
        'wallet_id',
        'qr_code_path',
        'activation_date',
        'expiry_date',
        'renewal_date',
        'status'
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'hashed_pin',
        'pin'
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array
     */
    protected $casts = [
        'last_pin_attempt' => 'datetime',
        'activation_date' => 'datetime',
        'expiry_date' => 'datetime',
        'renewal_date' => 'datetime',
        'pin_attempts' => 'integer'
    ];

    /**
     * Automatically hash the PIN when setting it
     *
     * @param  string  $value
     * @return void
     */
    public function setPinAttribute($value)
    {
        $this->attributes['pin'] = $value; // Temporary storage
        $this->attributes['hashed_pin'] = Hash::make($value);
    }

    /**
     * Check if the card is currently locked due to PIN attempts
     *
     * @return bool
     */
    public function getIsLockedAttribute()
    {
        return $this->pin_attempts >= 3 && 
               $this->last_pin_attempt &&
               $this->last_pin_attempt > now()->subHours(1);
    }

    /**
     * Reset PIN attempt counter
     *
     * @return void
     */
    public function resetPinAttempts()
    {
        $this->update([
            'pin_attempts' => 0,
            'last_pin_attempt' => null
        ]);
    }

    /**
     * Record a failed PIN attempt
     *
     * @return void
     */
    public function recordFailedAttempt()
    {
        $this->increment('pin_attempts');
        $this->update(['last_pin_attempt' => now()]);
    }

    /**
     * Relationship to the card owner
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function user()
    {
        return $this->belongsTo(User::class);
    }

    /**
     * Relationship to the associated wallet
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function wallet()
    {
        return $this->belongsTo(Wallet::class);
    }
}