const axios = require('axios');
require('dotenv').config();
const logger = require('./logger'); // Add this at the top

const instance = axios.create({
  baseURL: process.env.LARAVEL_API_URL,
  headers: {
    'Authorization': `Bearer ${process.env.LARAVEL_API_TOKEN}`,
    'Content-Type': 'application/json'
  }
});

module.exports = {
  activate: async (cardNumber, walletNumber, pin) => {
    logger.info('Card activation attempt', { 
      cardNumber,
      walletNumber,
      action: 'activate'
    });

    try {
      const response = await instance.post('/api/cards/activate', {
        card_number: cardNumber,
        wallet_number: walletNumber,
        pin: pin
      });
      
      logger.info('Card activated successfully', {
        cardNumber,
        status: response.status,
        data: response.data
      });
      
      return response.data;
    } catch (error) {
      logger.error('Card activation failed', {
        cardNumber,
        error: error.message,
        stack: error.stack
      });
      throw handleError(error);
    }
  },

  block: async (cardNumber, pin) => {
    logger.info('Card block attempt', { 
      cardNumber,
      action: 'block'
    });

    try {
      const response = await instance.post('/api/cards/block', {
        card_number: cardNumber,
        pin: pin
      });
      
      logger.info('Card blocked successfully', {
        cardNumber,
        status: response.status
      });
      
      return response.data;
    } catch (error) {
      logger.error('Card block failed', {
        cardNumber,
        error: error.message
      });
      throw handleError(error);
    }
  },

  reportStolen: async (cardNumber, reportDetails) => {
    logger.info('Stolen card report initiated', { 
      cardNumber,
      action: 'reportStolen'
    });

    try {
      const response = await instance.post('/api/cards/report-stolen', {
        card_number: cardNumber,
        report_details: reportDetails
      });
      
      logger.info('Stolen report submitted', {
        cardNumber,
        reference: response.data.reference_id
      });
      
      return response.data;
    } catch (error) {
      logger.error('Stolen report failed', {
        cardNumber,
        error: error.message
      });
      throw handleError(error);
    }
  }
};

// Existing error handler (update it with logging)
function handleError(error) {
  const logData = {
    status: error.response?.status || 500,
    url: error.config?.url,
    method: error.config?.method
  };

  if (error.response) {
    logger.error('API Error Response', {
      ...logData,
      data: error.response.data
    });
    return {
      status: error.response.status,
      data: error.response.data
    };
  }

  logger.error('Network Error', logData);
  return {
    status: 500,
    message: 'Network Error'
  };
}