<?php

use Illuminate\Support\Facades\Route;
use Illuminate\Support\Facades\Auth;

use App\Http\Controllers\KycController;
use App\Http\Controllers\CardController;
use App\Http\Controllers\AdminController;
use App\Http\Controllers\Auth\RegisterController;
use App\Http\Controllers\DonationController;
use App\Http\Controllers\VendorController;
use App\Http\Controllers\Admin\SectionController;
use App\Http\Controllers\Admin\UserController;
use App\Http\Controllers\AdminKycController;
use App\Http\Controllers\ProfileController;
use App\Http\Controllers\TransactionController;
use App\Http\Controllers\DonorController;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider and all of them will
| be assigned to the "web" middleware group.
|
*/

/**
 * Landing Page: Register page is the landing page.
 */
Route::get('/', [RegisterController::class, 'showRegistrationForm'])->name('landing');

/**
 * Custom Password Reset View
 */
Route::get('/password/reset', function () {
    return view('auth.passwords.reset');
})->name('password.request');

// Optional token-based reset route:
// Route::get('/password/reset/{token}', function ($token) {
//     return view('auth.passwords.reset', ['token' => $token]);
// })->name('password.reset');

/**
 * Authentication Routes
 */
Auth::routes();

/**
 * Dashboard Route (Authenticated)
 * If a vendor logs in, redirect them to the vendor dashboard.
 */

Route::middleware('auth')->group(function () {
    Route::get('/dashboard', function () {
        if (Auth::user()->role === 'vendor') {
            return redirect()->route('vendor.dashboard');
        }
        return view('dashboard');
    })->name('dashboard');
});


Route::middleware('auth')->group(function () {
    Route::get('/vendor/dashboard', function () {
        // Inline role check
        if (Auth::user()->role !== 'vendor') {
            abort(403, 'Unauthorized.');
        }
        return app(VendorController::class)->dashboard();
        // Alternatively, if you prefer returning a view directly:
        // return view('vendors.dashboard');
    })->name('vendor.dashboard');
});

/**
 * KYC Routes (Authenticated)
 */
Route::middleware('auth')->group(function () {
    Route::get('/kyc/create', [KycController::class, 'create'])->name('kyc.create');
    Route::post('/kyc/store', [KycController::class, 'store'])->name('kyc.store');
});

/**
 * Register Routes
 */
Route::get('/register', [RegisterController::class, 'showRegistrationForm'])->name('register');
Route::post('/register', [RegisterController::class, 'register']);

/**
 * Donation Routes (Authenticated)
 */
Route::middleware('auth')->group(function () {
    Route::get('/donate', [DonationController::class, 'showDonationForm'])->name('donations.create');
    Route::post('/donate', [DonationController::class, 'donate'])->name('donations.store');

    Route::get('/donations', [DonationController::class, 'showDonationOptions'])->name('donations.options');
    Route::get('/donations/card', [DonationController::class, 'showCardDonationForm'])->name('donations.card');
    Route::post('/donations/card', [DonationController::class, 'processCardDonation'])->name('donations.process.card');
    Route::get('/donations/bulk', [DonationController::class, 'showBulkDonationForm'])->name('donations.bulk');
    Route::post('/donations/bulk', [DonationController::class, 'processBulkDonation'])->name('donations.process.bulk');
});

/**
 * Card Routes (Common for Authenticated Users)
 */


Route::middleware('auth')->group(function () {
    Route::get('/card/activate', [CardController::class, 'showActivationForm'])->name('card.activate');
    Route::post('/card/activate/process', [CardController::class, 'processActivation'])->name('card.activate.process');

    // Card blocking routes
    Route::get('/card/block', [CardController::class, 'showBlockForm'])->name('card.block');
    Route::post('/card/block', [CardController::class, 'processBlock']);

    // Stolen reporting routes
    Route::get('/card/report-stolen', [CardController::class, 'showStolenReportForm'])->name('card.report-stolen');
    Route::post('/card/report-stolen', [CardController::class, 'processStolenReport']);
});




/**
 * Admin Routes (Requires Admin Role)
 */
Route::middleware(['auth', 'admin'])->group(function () {
    Route::get('/admin', [AdminController::class, 'index']);

    Route::get('/admin/dashboard', [AdminController::class, 'dashboard'])->name('admin.dashboard');
    Route::get('/admin/generate-cards', [AdminController::class, 'showGenerateCardsForm'])->name('cards.generate.form');
    Route::post('/admin/generate-cards', [AdminController::class, 'generateCards'])->name('cards.generate');
    Route::get('/admin/view-cards', [AdminController::class, 'viewCards'])->name('cards.view');
    Route::get('/admin/view-transactions', [AdminController::class, 'viewTransactions'])->name('transactions.view');

    Route::get('/admin/sections', [SectionController::class, 'index'])->name('sections.index');
    Route::get('/admin/sections/{section}/edit', [SectionController::class, 'edit'])->name('sections.edit');
    Route::put('/admin/sections/{section}', [SectionController::class, 'update'])->name('sections.update');

    Route::get('/admin/users', [UserController::class, 'index'])->name('users.index');
    Route::get('/admin/users/{user}/edit', [UserController::class, 'edit'])->name('users.edit');
    Route::put('/admin/users/{user}', [UserController::class, 'update'])->name('users.update');
    Route::delete('/admin/users/{user}', [UserController::class, 'destroy'])->name('users.destroy');

    Route::post('/vendor/kyc/submit', [VendorController::class, 'submitKyc'])->name('vendor.submitKyc');
});

Route::middleware(['auth'])->group(function () {
    // Vendor Settings Routes
    Route::prefix('vendor')->group(function () {
        // Profile Settings
        Route::get('/settings', [VendorController::class, 'settings'])->name('vendor.settings');
        Route::post('/update-profile', [VendorController::class, 'updateProfile'])->name('vendor.updateProfile');
        Route::post('/update-kyb', [VendorController::class, 'updateKYB'])->name('vendor.updateKYB');
        Route::post('/update-password', [VendorController::class, 'updatePassword'])->name('vendor.updatePassword');

        // Single form transaction
        Route::get('/transact', [VendorController::class, 'showTransactionForm'])
            ->name('vendor.transact.form');

        Route::post('/transact', [VendorController::class, 'processTransaction'])
            ->name('vendor.transact.process');
    });
});





/**
 * Profile Routes (Authenticated)
 */
Route::middleware('auth')->group(function () {
    Route::get('/profile/edit', [ProfileController::class, 'edit'])->name('profile.edit');
    Route::post('/profile/update', [ProfileController::class, 'update'])->name('profile.update');
});

/**
 * Vendor Redemption Routes (Authenticated)
 * These may be accessible to all vendors, but if you already have vendor-specific routes,
 * you can consolidate them into the vendor middleware group.
 */
/** 
 *Route::middleware(['auth', 'role.vendor'])->group(function () {
 *    // Vendor Dashboard & Redemption Routes (Only for Vendor Role)
 *    Route::get('/vendor/dashboard', [VendorController::class, 'dashboard'])->name('vendor.dashboard');
 *    Route::get('/vendor/redeem', [VendorController::class, 'showRedemptionForm'])->name('vendors.redeem');
 *    Route::post('/vendor/redeem', [VendorController::class, 'redeem'])->name('vendors.redeem.store');
 *});
 */

Route::middleware(['auth'])->group(function () {
    Route::get('/vendor/kyc', [VendorController::class, 'showKycPage'])->name('vendor.kyc');
});

Route::middleware('auth')->group(function () {
    // Display the transaction form (GET)
    //Route::post('/transaction', [TransactionController::class, 'showTransactionForm'])->name('transaction.form');

    // Process the transaction form (POST)
    //Route::post('/transaction/process', [TransactionController::class, 'processTransaction'])->name('transaction.process');
});




Route::middleware(['auth', 'verified', 'vendor'])->group(function () {
    // Change from:
    Route::get('/transactions/create', [TransactionController::class, 'showTransactionForm'])
        ->name('transactions.create');

    // To either (Option A - Singular):
    Route::get('/transaction/create', [TransactionController::class, 'showTransactionForm'])
        ->name('transaction.create');

    // OR (Option B - Plural - recommended for RESTful conventions):
    Route::get('/transactions/create', [TransactionController::class, 'showTransactionForm'])
        ->name('transactions.create');
});

Route::get('/transactions', [TransactionController::class, 'index'])->middleware('auth');

// routes/web.php
Route::middleware(['auth', 'role:donor'])->group(function () {
    Route::get('/donor/fund', [DonorController::class, 'showFundingForm'])->name('donor.fund.form');
    Route::post('/donor/fund', [DonorController::class, 'fundWallet'])->name('donor.fund');
    Route::get('/donor/donate', [DonorController::class, 'showDonationForm'])->name('donor.donate.form');
    Route::post('/donor/donate', [DonorController::class, 'processDonation'])->name('donor.donate');
});

Route::get('/logs', function() {
    return response()->file(storage_path('logs/api.log'));
});



// Terms & Privacy Routes (Public)
Route::get('/terms', function () {
    return view('terms');
})->name('terms');
Route::get('/privacy', function () {
    return view('privacy');
})->name('privacy');
